#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "vegl.h"

HINSTANCE		g_hInst;				// current instance
HWND			g_hMainWnd;				// Display Window

//Definitions
LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);
void render_vegl();
void transfer_vegl(const BYTE * lpData, DWORD nSize);

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	g_hInst = hInstance;
	//Create the main window
    WNDCLASS wndClass = { CS_HREDRAW | CS_VREDRAW, WndProc, 0, 0, g_hInst,
                          NULL, LoadCursor(NULL, MAKEINTRESOURCE(IDC_ARROW)), 
                          (HBRUSH)GetStockObject(BLACK_BRUSH),
                          NULL, TEXT("veGL") };
    RegisterClass( &wndClass );
    g_hMainWnd = CreateWindowEx(0, TEXT("veGL"), NULL,
                              WS_POPUP|WS_VISIBLE|WS_CLIPCHILDREN|WS_CLIPSIBLINGS, 0, 0, 768, 224, 
							  0L, 0L, g_hInst, 0L );
	if( !g_hMainWnd )
	{
		MessageBox(NULL,"Win32 fatal error","veGL",MB_OK|MB_ICONERROR);
		return 0;
	}

	//Now manage the created window as an OpenGL compatible device for veGL.
	if( !veglOpen(g_hMainWnd) )
	{
		MessageBox(NULL,"veglOpen fatal error","veGL",MB_OK|MB_ICONERROR);
		PostQuitMessage(0);
	}
	else
	{
		//Needed elements for our example, can be different according to the purpose
	    glClearColor( 0.0, 0.0, 0.0, 1.0 );		//define the clear color
		glClearDepth( 1.0 );					//define the depth to clear in every call.
		glEnable(GL_CULL_FACE);					//Don't render backfaces (requieres normal data).
		glFrontFace(GL_CCW);					//The frontfaces are in CounterClockWise order.
		glEnable(GL_NORMALIZE);					//Perform normilize
		glDepthFunc(GL_LEQUAL);					//Depth test function <=
		GLfloat mat[4] = {0.5f,0.5f,0.5f,1.0f};	//Standard material for our demo
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat);

		glEnable(GL_DEPTH_TEST);				//Use render with depth test
		glEnable(GL_LIGHTING);					//Use lights in render
		glEnable(GL_LIGHT0);					//Use light 0 in render
		GLfloat lightpos[4] = {3.0, 0.0, 3.0, 3.0};		//Position of light 0
		GLfloat lightcol[4] = {1.0, 1.0, 1.0, 1.0};		//Color of light 0
		GLfloat lightdir[3] = {-1.0, -1.0, 1.0};		//Direction of light 0
		glLightfv(GL_LIGHT0, GL_POSITION, lightpos);    //Defining values
		glLightfv(GL_LIGHT0, GL_AMBIENT, lightcol);
		glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, lightdir);
	}
    
	while(true)
    {
		//Render instructions
		{
			glClearColor(0,0,0,1);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			if( !veglRender(render_vegl) )
				MessageBox(NULL,"veglRender fatal error","veGL",MB_OK|MB_ICONERROR);
		}

		if( !veglTransfer(transfer_vegl) )
			MessageBox(NULL,"veglTransfer fatal error","veGL",MB_OK|MB_ICONERROR);

        if (!PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
            continue;
        if (msg.message == WM_QUIT)
            break;
        TranslateMessage (&msg);
		DispatchMessage (&msg);
    }

	//Close all the veGL interface
	if( !veglClose() )
		MessageBox(NULL,"veglClose fatal error","veGL",MB_OK|MB_ICONERROR);
	return msg.wParam;
}

//function called when the render is processed
void render_vegl()
{
	static float key = 0.0f;

    glPushMatrix();
		glRotatef(key+=1.0f,1,0,0);
		glRotatef(key,0,0,1);
		//Render a simple cube
		glBegin(GL_QUAD_STRIP);
			glNormal3f(0,0,1);
			glVertex3f(-0.5f, 0.5f, 0.5f); glVertex3f(-0.5f, -0.5f, 0.5f);
			glVertex3f(0.5f, 0.5f, 0.5f);  glVertex3f(0.5f, -0.5f, 0.5f);
			glNormal3f(0,0,-1);
			glVertex3f(0.5f, 0.5f, -0.5f); glVertex3f(0.5f, -0.5f, -0.5f);
			glVertex3f(-0.5f, 0.5f, -0.5f); 
			glNormal3f(-1,0,0);
			glVertex3f(-0.5f, -0.5f,  -0.5f);
			glVertex3f(-0.5f, 0.5f, 0.5f); glVertex3f(-0.5f, -0.5f, 0.5f);
		glEnd();
        glBegin(GL_QUADS);
			glNormal3f(0,1,0);
			glVertex3f(-0.5f, 0.5f, 0.5f); glVertex3f(0.5f, 0.5f, 0.5f);
			glVertex3f(0.5f, 0.5f, -0.5f); glVertex3f(-0.5f, 0.5f, -0.5f);
        glEnd();
        glBegin(GL_QUADS);
			glNormal3f(0,-1,0);
            glVertex3f(0.5f, -0.5f, 0.5f); glVertex3f(-0.5f, -0.5f, 0.5f);
			glVertex3f(-0.5f,-0.5f,-0.5f); glVertex3f(0.5f, -0.5f,  -0.5f);
        glEnd();
    glPopMatrix();
}

//function called when the transfer is processed, maybe used to process personal feedback.
void transfer_vegl(const BYTE* lpData, DWORD nSize)
{
}

//usual callback window procedure
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
        case WM_DESTROY:
			PostQuitMessage(0);
            return 0L;
        case WM_KEYDOWN:
            break;
        case WM_SETCURSOR:
            SetCursor(NULL);
            return TRUE;
   }
   return DefWindowProc(hWnd, message, wParam, lParam);
}

